#include "SOMO.h"
#include <EEPROM.h>

#define DEBUG

#define SOMO_CLOCK_PIN 4
#define SOMO_DATA_PIN 3
#define SOMO_BUSY_PIN 2
#define SOMO_RESET_PIN 0
#define BUTTON_PIN 7
#define BUZZER_PIN 8
#define RED_PIN 5
#define DIP0_PIN 13
#define DIP1_PIN 12
#define DIP2_PIN 11
#define DIP3_PIN 10
#define DIP4_PIN 9
#define DIP5_PIN A0
#define DIP6_PIN A1
#define DIP7_PIN A2

#define DIGITAL_DIP_COUNT 5
#define ANALOG_DIP_COUNT 3

int digitalDipSwitches[] = {DIP0_PIN, DIP1_PIN, DIP2_PIN, DIP3_PIN, DIP4_PIN};
int analogDipSwitches[] = {DIP5_PIN, DIP6_PIN, DIP7_PIN};

unsigned int volLevel=0x0005;
int Song; // current Song
// used to neutralize long presses
boolean buttonPressed = false;
int trackCount = 0;

int colorIdx = 0;
const int colorsWhilePlaying[][3] = {{0,0,100}, {0,100,100}, {0,100,0}, {100,100,0}, {100,0,0}, {100,0,100}};

void setup(){
#ifdef DEBUG  
  Serial.begin(9600);
#endif
  Init(SOMO_CLOCK_PIN, SOMO_DATA_PIN, SOMO_BUSY_PIN, SOMO_RESET_PIN);
  pinMode(BUTTON_PIN, INPUT);
  pinMode(BUZZER_PIN, OUTPUT);
  for(int i = 0; i <  DIGITAL_DIP_COUNT; ++i) {
    pinMode(digitalDipSwitches[i], OUTPUT);
    digitalWrite(digitalDipSwitches[i], HIGH);
  }
  digitalWrite(BUTTON_PIN, HIGH);
  randomSeed(analogRead(0));
  // initilize and clear EEPROM on first activation and when restarting with button pressed (EEPROM reset)
  if((digitalRead(BUTTON_PIN) == LOW) || !EEPROMEverInitialized()) {
    initializeEEPROM();
  }
  transition(0,0,0,100,100,100, 100, 3, false, false);
  transition(0,0,0, 100, 3, false, false);
  for(int i = 0; i <  DIGITAL_DIP_COUNT; ++i) {
    trackCount += (digitalRead(digitalDipSwitches[i]) == HIGH ? 0 : 1) << i;
  }
  for(int i = 0; i < ANALOG_DIP_COUNT; ++i) {
    int val = analogRead(analogDipSwitches[i]);
    trackCount += ((val > 512) ? 0 : 1) << (DIGITAL_DIP_COUNT + i);
  }
  print("number of tracks detected: "); println(trackCount);
  println("setup() complete");
}

void loop() {
  if(digitalRead(BUTTON_PIN) == LOW) {
    if(buttonPressed == false) {
      buttonPressed = true;
      if(IsBusy()) {
        StopPlay();
      }
      else {
        playRandomTrack();
      }
    }
  }
  else {
    buttonPressed = false;
  }
  if(IsBusy()) {
    if(!updateTransition()) {
      transition(colorsWhilePlaying[colorIdx][0],colorsWhilePlaying[colorIdx][1],colorsWhilePlaying[colorIdx][2], 100, 2, false, true);
      colorIdx = (colorIdx + 1) % 6;
    }
  }
  else {
    ledsOff(false);
  }
}



/**********************************************************************************
 * EEPROM Management
 **********************************************************************************/

boolean EEPROMEverInitialized() {
  for(int i = 0; i < 10; ++i) {
    if(EEPROM.read(i) != 37 + i) {
      println("EEPROM never initialized");
      return false;
    }
  }
  println("EEPROM already initialized");
  return true;
}


void initializeEEPROM() {
  println("initializing EEPROM");
  for(int i = 0; i < 3; ++i) {
    setRGB(255,0,0,false);
    tone(BUZZER_PIN, 1000);
    delay(100);
    setRGB(0,0,0,false);
    noTone(BUZZER_PIN);
    delay(50);
  }
  
  for(int i = 0; i < 1024; ++i) {
    tone(BUZZER_PIN, 100 + i);
    if(i < 10) {
      EEPROM.write(i, 37 + i);
    }
    else {
      EEPROM.write(i, 0);
    }
  }
  noTone(BUZZER_PIN);
}

boolean unplayedTrackExist() {
  print("hasUnplayedTracks() called, returning: ");
  for(int i = 0; i < trackCount; ++i) {
    if(!trackPlayed(i)) {
      println("true");
      return true;
    }
  }
  println("false");
  return false;
}

boolean trackPlayed(int trackNo) {
  return (EEPROM.read(10 + trackNo) == 1);
}

void setTrackPlayed(int trackNo) {
  EEPROM.write(10 + trackNo, 1);
}

/**********************************************************************************
 * Sound Management
 **********************************************************************************/

void playRandomTrack() {
  if(!unplayedTrackExist()) {
    initializeEEPROM();
  }
  
  print("selecting random track");
  int i;
  do {
    i = random(trackCount);
  }
  while(trackPlayed(i));
  
  print("playing random track...");
  println(i);
  PlaySong(i);
  setTrackPlayed(i);
}

void incVol()
{ 
  if (volLevel  < SOMO_MAX_VOLUME){ 
    volLevel++;
    print("Increase volume to: ");
    println(volLevel);
    SetVolume(volLevel);
  }

}

void decVol()
{
  if (volLevel > SOMO_MIN_VOLUME){ 
    volLevel--;
    print("Decrease volume to: ");
    println(volLevel);
    SetVolume(volLevel);
  }
}

/**********************************************************************************
 * RGB LED Management
 **********************************************************************************/

int m_redStart;
int m_greenStart;
int m_blueStart;
int m_redEnd;
int m_greenEnd;
int m_blueEnd;
int m_count;
int m_steps;
int m_delayy;
boolean m_commonAnode;
int m_redRange;
int m_greenRange;
int m_blueRange;
int m_redDir;
int m_greenDir;
int m_blueDir;
int m_red;
int m_green;
int m_blue;


void transition(int redEnd, int greenEnd, int blueEnd, int steps, int delayy, boolean commonAnode, boolean async) {
  transition(
    m_commonAnode ? (255 - m_redEnd) : m_redEnd, 
    m_commonAnode ? (255 - m_greenEnd) : m_greenEnd,
    m_commonAnode ? (255 - m_blueEnd) : m_blueEnd, 
    redEnd,  greenEnd, blueEnd,
    steps, delayy, commonAnode, async);
}

void transition(int redStart, int greenStart, int blueStart, int redEnd, int greenEnd, int blueEnd, int steps, int delayy, boolean commonAnode, boolean async) {
  m_redStart = commonAnode ? 255-redStart : redStart;
  m_redEnd = commonAnode ? 255-redEnd : redEnd;
//  m_greenStart = commonAnode ? 255-greenStart : greenStart;
//  m_greenEnd = commonAnode ? 255-greenEnd : greenEnd;
//  m_blueStart = commonAnode ? 255-blueStart : blueStart;
//  m_blueEnd = commonAnode ? 255-blueEnd : blueEnd;
  m_redRange = abs(redStart - redEnd);
//  m_greenRange = abs(greenStart - greenEnd);
//  m_blueRange = abs(blueStart - blueEnd);
  m_redDir = (m_redStart < m_redEnd) ? 1 : -1;
//  m_greenDir = (m_greenStart < m_greenEnd) ? 1 : -1;
//  m_blueDir = (m_blueStart < m_blueEnd) ? 1 : -1;
  m_steps = steps;
  m_delayy = delayy;
  m_commonAnode = commonAnode;
  if(!async) {
    for(m_count = 1; m_count <= m_steps; ++m_count) {
      m_red = m_redStart + m_redRange * m_redDir * m_count / m_steps;
      analogWrite(RED_PIN, m_red);
//      m_green = m_greenStart + m_greenRange * m_greenDir * m_count / m_steps;
//      analogWrite(GREEN_PIN, m_green);
//      m_blue = m_blueStart + m_blueRange * m_blueDir * m_count / m_steps;
//      analogWrite(BLUE_PIN, m_blue);
      delay(m_delayy);
    }
  }
  else {
    m_count = 1;
  }
}


boolean updateTransition() {
  if(m_count <= m_steps) {
    m_red = m_redStart + m_redRange * m_redDir * m_count / m_steps;
    analogWrite(RED_PIN, m_red);
//    m_green = m_greenStart + m_greenRange * m_greenDir * m_count / m_steps;
//    analogWrite(GREEN_PIN, m_green);
//    m_blue = m_blueStart + m_blueRange * m_blueDir * m_count / m_steps;
//    analogWrite(BLUE_PIN, m_blue);
    delay(m_delayy);
    ++m_count;
    return true;
  }
  else {
    return false;
  }
}

boolean ledsOff(boolean commonAnode) {
  setRGB(0,0,0,commonAnode);
}

void setRGB(int r, int g, int b, boolean commonAnode) {
  analogWrite(RED_PIN, commonAnode ? 255-r : r);
//  analogWrite(GREEN_PIN, commonAnode ? 255-g : g);
//  analogWrite(BLUE_PIN, commonAnode ? 255-b : b);
}  

void println(char* msg) {
#ifdef DEBUG  
  Serial.println(msg);
#endif
}

void print(char* msg) {
#ifdef DEBUG  
  Serial.print(msg);
#endif
}

void println(int i) {
#ifdef DEBUG  
  Serial.println(i);
#endif
}

void print(int i) {
#ifdef DEBUG  
  Serial.print(i);
#endif
}

